#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

public enum DisplayTypeTCTrendingTSF
{
	Indicator,
	PaintBars,
	All
}

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Copyright TradingCoders.com 2009.
	/// DO NOT REDISTRIBUTE. DO NOT REMOVE TradingCoders.com COPYRIGHT NOTICE. 
	/// NOT FOR RESALE.
    /// </summary>
    [Description("Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com")]
    //[Gui.Design.DisplayName("TC_TrendingTSF: TradingCoders.com")]
	public class TCTrendingTSF : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int 	tSFPeriod = 50; // Default setting for TSFPeriod
            private int 	tSFForecast = 3; // Default setting for TSFForecast
			private DisplayTypeTCTrendingTSF	display = DisplayTypeTCTrendingTSF.All;
			private Color upColor = Color.Blue;
			private Color downColor = Color.Red;
			private int countertrendOpacity = 4;
			private string alertSound = "Alert4.wav";
			private bool alertEnabled = true;
        // User defined variables (add any user defined variables below)
			private double TSFValue = 0;
			private bool	TrendFlag = true;	// true = UP, false = down;
			private bool	TrendChange = true;
			private int		GrID = 0;
			private int		FlexiLength = 0;
			private double	TSFValueMem = 0;
			private BoolSeries signalDirection;
		/// <summary>
		/// +10 is first and second bar in an uptrend, +1 is any other bar in an uptrend, -10 and -1 likewise for downtrends
		/// </summary>
			private DataSeries  mA_Signals;
		
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Name = "TC_TrendingTSF: TradingCoders.com";
            Add(new Plot(Color.FromKnownColor(KnownColor.RoyalBlue), PlotStyle.Line, "TrendingTSF"));
			Plots[0].Pen.Width = 3;
            Overlay				= true;
            PriceTypeSupported	= true;
			BarsRequired = 1;
			
			signalDirection = new BoolSeries(this);
			mA_Signals = new DataSeries(this);
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar<5) return;
			

			// calculation logic
			if (CurrentBar>TSFPeriod+1)
			{
				TSFValueMem = TSF(Input,TSFForecast,TSFPeriod)[1];	
				TSFValue = TSF(Input,TSFForecast,TSFPeriod)[0];
				TrendChange = false;
				TrendingTSF.Set(TSFValue);
				TrendFlag = SignalDirection[1];
				
				if ((Close[0] > TSFValue) && (TSFValue > TSFValueMem))
				{
					if (!TrendFlag) // TrendFlag
						TrendChange = true;
					TrendFlag = true;
					
				}
				else if ((Close[0] < TSFValue) && (TSFValue < TSFValueMem))
				{
					if (TrendFlag) //(TrendFlag)
						TrendChange = true;
					TrendFlag = false;
					
				}
				
				// publish public values
				if (TrendFlag)
					signalDirection.Set(true);
				else
					signalDirection.Set(false);
				// Market Analyzer compatible series, 
				if 	    (TrendFlag && signalDirection[1])
					mA_Signals.Set(+1);
				else if (TrendFlag && (!signalDirection[1]))
					mA_Signals.Set(+10);
				else if ((!TrendFlag) && (!signalDirection[1]))
					mA_Signals.Set(-1);
				else if ((!TrendFlag) && signalDirection[1])
					mA_Signals.Set(-10);
				
				
				
				// price bar color
				if (TrendFlag)
				{
					if (Display != DisplayTypeTCTrendingTSF.Indicator)
					{
						BarColor = UpColor;
						CandleOutlineColor = UpColor;
						if (Close[0]<Open[0])
						{
							//CandleOutlineColor = DownColor;
							BarColor = Color.FromArgb((int)(25.5*CountertrendOpacity),UpColor);
						}

					}
				}
					else
				{
					if (Display != DisplayTypeTCTrendingTSF.Indicator)
					{
						BarColor = DownColor;
						CandleOutlineColor = DownColor;
						if (Close[0] > Open[0])
						{
							//CandleOutlineColor = UpColor;
							BarColor = Color.FromArgb((int)(25.5*CountertrendOpacity),DownColor);
						}

					}
				}
			
				// indicator line display
				if (Display != DisplayTypeTCTrendingTSF.PaintBars) 
				{
					if (TrendFlag)	
					{
						PlotColors[0][0] = UpColor;
					}
					else if (!TrendFlag)
					{
						PlotColors[0][0] = DownColor;
					}
					
					
				}
				
				// alerts
				int az = (Historical || CalculateOnBarClose) ? 0 : 1;
				if (	FirstTickOfBar && CurrentBar > TSFPeriod &&
						AlertEnabled && SignalDirection[az] != SignalDirection[az+1]	)
				{
					if (SignalDirection[az])
						Alert("TSFAlert",Priority.Medium,"TCTrendingTSF LONG",AlertSound,15,Color.PaleGreen,Color.Black);
					else 
						
						Alert("TSFAlert",Priority.Medium,"TCTrendingTSF SHORT",AlertSound,15,Color.Pink,Color.Black);
				}

			}
            
        }
		
		public override string ToString()
		{
			return this.Name;
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TrendingTSF
        {
            get { return Values[0]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA_Signals
        {
            get { 
				Update();
				return mA_Signals; }
        }

		[Browsable(false)]
		public BoolSeries SignalDirection
		{
			
			get { 
				Update();
				return signalDirection; }			
		}
		
// ---------------------

        [Description("TSF Period")]
        [GridCategory("Parameters")]
        public int TSFPeriod
        {
            get { return tSFPeriod; }
            set { tSFPeriod = Math.Max(1, value); }
        }

        [Description("TSF Forecast Bars")]
        [GridCategory("Parameters")]
        public int TSFForecast
        {
            get { return tSFForecast; }
            set { tSFForecast = Math.Max(0, value); }
        }
		
		[Description("Display Type")]
        [GridCategory("Display")]
        public DisplayTypeTCTrendingTSF Display
        {
            get { return display; }
            set { display = value; }
        }
		
		[Description("Counter-trend Candle body Opacity, 0 to 10")]
        [GridCategory("Display")]
        public int CountertrendOpacity
        {
            get { return countertrendOpacity; }
            set { countertrendOpacity = Math.Max(0,Math.Min(10,value)); }
        }
		
		[XmlIgnore()]
        [Description("Color for Up Trend")]
        [GridCategory("Display")]
        public Color UpColor
        {
            get { return upColor; }
            set { upColor = value; }
        }
		
		// Serialize our Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for Down Trend")]
        [GridCategory("Display")]
        public Color DownColor
        {
            get { return downColor; }
            set { downColor = value; }
        }
		
		// Serialize our Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		
		[Description("Either built-int sound or absolute file path")]
        [Category("Alerts")]
        public string AlertSound
        {
            get { return alertSound; }
            set { alertSound = value; }
        }
		
		[Description("Enable alerts?")]
        [Category("Alerts")]
        public bool AlertEnabled
        {
            get { return alertEnabled; }
            set { alertEnabled = value; }
        }
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TCTrendingTSF[] cacheTCTrendingTSF = null;

        private static TCTrendingTSF checkTCTrendingTSF = new TCTrendingTSF();

        /// <summary>
        /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com
        /// </summary>
        /// <returns></returns>
        public TCTrendingTSF TCTrendingTSF(int countertrendOpacity, DisplayTypeTCTrendingTSF display, Color downColor, int tSFForecast, int tSFPeriod, Color upColor)
        {
            return TCTrendingTSF(Input, countertrendOpacity, display, downColor, tSFForecast, tSFPeriod, upColor);
        }

        /// <summary>
        /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com
        /// </summary>
        /// <returns></returns>
        public TCTrendingTSF TCTrendingTSF(Data.IDataSeries input, int countertrendOpacity, DisplayTypeTCTrendingTSF display, Color downColor, int tSFForecast, int tSFPeriod, Color upColor)
        {
            if (cacheTCTrendingTSF != null)
                for (int idx = 0; idx < cacheTCTrendingTSF.Length; idx++)
                    if (cacheTCTrendingTSF[idx].CountertrendOpacity == countertrendOpacity && cacheTCTrendingTSF[idx].Display == display && cacheTCTrendingTSF[idx].DownColor == downColor && cacheTCTrendingTSF[idx].TSFForecast == tSFForecast && cacheTCTrendingTSF[idx].TSFPeriod == tSFPeriod && cacheTCTrendingTSF[idx].UpColor == upColor && cacheTCTrendingTSF[idx].EqualsInput(input))
                        return cacheTCTrendingTSF[idx];

            lock (checkTCTrendingTSF)
            {
                checkTCTrendingTSF.CountertrendOpacity = countertrendOpacity;
                countertrendOpacity = checkTCTrendingTSF.CountertrendOpacity;
                checkTCTrendingTSF.Display = display;
                display = checkTCTrendingTSF.Display;
                checkTCTrendingTSF.DownColor = downColor;
                downColor = checkTCTrendingTSF.DownColor;
                checkTCTrendingTSF.TSFForecast = tSFForecast;
                tSFForecast = checkTCTrendingTSF.TSFForecast;
                checkTCTrendingTSF.TSFPeriod = tSFPeriod;
                tSFPeriod = checkTCTrendingTSF.TSFPeriod;
                checkTCTrendingTSF.UpColor = upColor;
                upColor = checkTCTrendingTSF.UpColor;

                if (cacheTCTrendingTSF != null)
                    for (int idx = 0; idx < cacheTCTrendingTSF.Length; idx++)
                        if (cacheTCTrendingTSF[idx].CountertrendOpacity == countertrendOpacity && cacheTCTrendingTSF[idx].Display == display && cacheTCTrendingTSF[idx].DownColor == downColor && cacheTCTrendingTSF[idx].TSFForecast == tSFForecast && cacheTCTrendingTSF[idx].TSFPeriod == tSFPeriod && cacheTCTrendingTSF[idx].UpColor == upColor && cacheTCTrendingTSF[idx].EqualsInput(input))
                            return cacheTCTrendingTSF[idx];

                TCTrendingTSF indicator = new TCTrendingTSF();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.CountertrendOpacity = countertrendOpacity;
                indicator.Display = display;
                indicator.DownColor = downColor;
                indicator.TSFForecast = tSFForecast;
                indicator.TSFPeriod = tSFPeriod;
                indicator.UpColor = upColor;
                Indicators.Add(indicator);
                indicator.SetUp();

                TCTrendingTSF[] tmp = new TCTrendingTSF[cacheTCTrendingTSF == null ? 1 : cacheTCTrendingTSF.Length + 1];
                if (cacheTCTrendingTSF != null)
                    cacheTCTrendingTSF.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTCTrendingTSF = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TCTrendingTSF TCTrendingTSF(int countertrendOpacity, DisplayTypeTCTrendingTSF display, Color downColor, int tSFForecast, int tSFPeriod, Color upColor)
        {
            return _indicator.TCTrendingTSF(Input, countertrendOpacity, display, downColor, tSFForecast, tSFPeriod, upColor);
        }

        /// <summary>
        /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com
        /// </summary>
        /// <returns></returns>
        public Indicator.TCTrendingTSF TCTrendingTSF(Data.IDataSeries input, int countertrendOpacity, DisplayTypeTCTrendingTSF display, Color downColor, int tSFForecast, int tSFPeriod, Color upColor)
        {
            return _indicator.TCTrendingTSF(input, countertrendOpacity, display, downColor, tSFForecast, tSFPeriod, upColor);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TCTrendingTSF TCTrendingTSF(int countertrendOpacity, DisplayTypeTCTrendingTSF display, Color downColor, int tSFForecast, int tSFPeriod, Color upColor)
        {
            return _indicator.TCTrendingTSF(Input, countertrendOpacity, display, downColor, tSFForecast, tSFPeriod, upColor);
        }

        /// <summary>
        /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com
        /// </summary>
        /// <returns></returns>
        public Indicator.TCTrendingTSF TCTrendingTSF(Data.IDataSeries input, int countertrendOpacity, DisplayTypeTCTrendingTSF display, Color downColor, int tSFForecast, int tSFPeriod, Color upColor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TCTrendingTSF(input, countertrendOpacity, display, downColor, tSFForecast, tSFPeriod, upColor);
        }
    }
}
#endregion
