#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion


// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators 
{
    /// <summary>
    /// Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Copyright TradingCoders.com 2009.
	/// DO NOT REDISTRIBUTE. DO NOT REMOVE TradingCoders.com COPYRIGHT NOTICE. 
	/// NOT FOR RESALE.
    /// </summary>
    [Description("Uses a Time Series Forecast line, and its relative position with price to declare a Trend direction. Courtesy of TradingCoders.com")]
	public class TCTrendingTSF : Indicator
    {			
		public enum DisplayTypeTCTrendingTSF
		{
			Indicator,
			PaintBars,
			All
		}

        #region Variables
		
        // User defined variables (add any user defined variables below)
			private double TSFValue = 0;
			private bool	TrendFlag = true;	// true = UP, false = down;
			private bool	TrendChange = true;
			private int		GrID = 0;
			private int		FlexiLength = 0;
			private double	TSFValueMem = 0;
			private Series<bool> signalDirection;
		/// <summary>
		/// +10 is first and second bar in an uptrend, +1 is any other bar in an uptrend, -10 and -1 likewise for downtrends
		/// </summary>
			private Series<double>  mA_Signals;
		
			Brush lightUpColor, lightDownColor;
		
        #endregion

		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Initialize();	// call the original NT7 Initialize()
			}
			else if (State == State.Configure)
			{
				OnStartUp();	// call the original NT7 OnStartUp()
			}
		}
		
        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected void Initialize()
        {
			Name = "TC_TrendingTSF: TradingCoders.com";
            AddPlot(new Stroke(Brushes.RoyalBlue,3), PlotStyle.Line, "TrendingTSF");
            IsOverlay				= true;
			BarsRequiredToPlot = 1;
			
			signalDirection = new Series<bool>(this);
			mA_Signals = new Series<double>(this);
			
			TSFPeriod = 50; // Default setting for TSFPeriod
            TSFForecast = 3; // Default setting for TSFForecast
			Display = DisplayTypeTCTrendingTSF.All;
			UpColor = Brushes.Blue;
			DownColor = Brushes.Red;
			CountertrendOpacity = 4;
			AlertSound = "Alert4.wav";
			AlertEnabled = true;
			
        }
		
		protected void OnStartUp()
		{
			lightUpColor = UpColor.Clone();
			lightUpColor.Opacity = CountertrendOpacity*0.1;
			lightUpColor.Freeze();
			
			lightDownColor = DownColor.Clone();
			lightDownColor.Opacity = CountertrendOpacity*0.1;
			lightDownColor.Freeze();
		}
		
		public override string DisplayName
		{
			get { return "TC_TrendingTSF("+TSFPeriod+","+TSFForecast+")"; }
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar<5) return;
			

			// calculation logic
			if (CurrentBar>TSFPeriod+1)
			{
				TSFValueMem = TSF(Input,TSFForecast,TSFPeriod)[1];	
				TSFValue = TSF(Input,TSFForecast,TSFPeriod)[0];
				TrendChange = false;
				TrendingTSF[0] = (TSFValue);
				TrendFlag = SignalDirection[1];
				
				if ((Close[0] > TSFValue) && (TSFValue > TSFValueMem))
				{
					if (!TrendFlag) // TrendFlag
						TrendChange = true;
					TrendFlag = true;
					
				}
				else if ((Close[0] < TSFValue) && (TSFValue < TSFValueMem))
				{
					if (TrendFlag) //(TrendFlag)
						TrendChange = true;
					TrendFlag = false;
					
				}
				
				// publish public values
				if (TrendFlag)
					signalDirection[0] = (true);
				else
					signalDirection[0] = (false);
				// Market Analyzer compatible series, 
				if 	    (TrendFlag && signalDirection[1])
					mA_Signals[0] = (+1);
				else if (TrendFlag && (!signalDirection[1]))
					mA_Signals[0] = (+10);
				else if ((!TrendFlag) && (!signalDirection[1]))
					mA_Signals[0] = (-1);
				else if ((!TrendFlag) && signalDirection[1])
					mA_Signals[0] = (-10);
				
				
				
				// price bar color
				if (TrendFlag)
				{
					if (Display != DisplayTypeTCTrendingTSF.Indicator)
					{
						BarBrush = UpColor;
						CandleOutlineBrush = UpColor;
						if (Close[0]<Open[0])
						{
							BarBrush = lightUpColor;
						}

					}
				}
					else
				{
					if (Display != DisplayTypeTCTrendingTSF.Indicator)
					{
						BarBrush = DownColor;
						CandleOutlineBrush = DownColor;
						if (Close[0] > Open[0])
						{
							BarBrush = lightDownColor;
						}

					}
				}
			
				// indicator line display
				if (Display != DisplayTypeTCTrendingTSF.PaintBars) 
				{
					if (TrendFlag)	
					{
						PlotBrushes[0][0] = UpColor;
					}
					else if (!TrendFlag)
					{
						PlotBrushes[0][0] = DownColor;
					}
					
					
				}
				
				// alerts
				int az = (State==State.Historical || Calculate==Calculate.OnBarClose) ? 0 : 1;
				if (	IsFirstTickOfBar && CurrentBar > TSFPeriod &&
						AlertEnabled && SignalDirection[az] != SignalDirection[az+1]	)
				{
					if (SignalDirection[az])
						Alert("TSFAlert",Priority.Medium,"TCTrendingTSF LONG",NinjaTrader.Core.Globals.InstallDir+@"\sounds\"+AlertSound,15,Brushes.PaleGreen,Brushes.Black);
					else 
						
						Alert("TSFAlert",Priority.Medium,"TCTrendingTSF SHORT",NinjaTrader.Core.Globals.InstallDir+@"\sounds\"+AlertSound,15,Brushes.Pink,Brushes.Black);
				}

			}
            
        }
		
		public override string ToString()
		{
			return this.Name;
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> TrendingTSF
        {
            get { return Values[0]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MA_Signals
        {
            get { 
				Update();
				return mA_Signals; }
        }

		[Browsable(false)]
		public Series<bool> SignalDirection
		{
			
			get { 
				Update();
				return signalDirection; }			
		}
		
		// ---------------------

		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(Description = "TSF Period",
		 ResourceType = typeof(Custom.Resource), Name = "TSF Period", GroupName = "NinjaScriptParameters", Order = 0)]
		public int TSFPeriod
		{ get; set; }
       
		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(Description = "TSF Forecast Bars",
		 ResourceType = typeof(Custom.Resource), Name = "TSF Forecast", GroupName = "NinjaScriptParameters", Order = 1)]
		public int TSFForecast
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Description = "Display type",
		 ResourceType = typeof(Custom.Resource), Name = "Display", GroupName = "Display", Order = 0)]
		public NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF Display
		{ get; set; }
		
		
		[Range(0, 10), NinjaScriptProperty]
		[Display(Description = "Counter-trend Candle body Opacity, 0 to 10",
		 ResourceType = typeof(Custom.Resource), Name = "Countertrend Opacity", GroupName = "Display", Order = 1)]
		public int CountertrendOpacity
		{ get; set; }
       
		
		
		[XmlIgnore()]
        //[NinjaScriptProperty]
		[Display(Description = "Color for uptrend",
		 ResourceType = typeof(Custom.Resource), Name = "Up Color", GroupName = "Display", Order = 2)]
		public Brush UpColor
		{ get; set; }
		
		// Serialize our Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return Serialize.BrushToString(UpColor); }
			set { UpColor = Serialize.StringToBrush(value); }
		}
		
		
		[XmlIgnore()]
        //[NinjaScriptProperty]
		[Display(Description = "Color for downtrend",
		 ResourceType = typeof(Custom.Resource), Name = "Down Color", GroupName = "Display", Order = 3)]
		public Brush DownColor
		{ get; set; }
		
		// Serialize our Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return Serialize.BrushToString(DownColor); }
			set { DownColor = Serialize.StringToBrush(value); }
		}
		
		
		//[NinjaScriptProperty]
		[Display(Description = "Built in Ninja sound",
		 ResourceType = typeof(Custom.Resource), Name = "Alert Sound", GroupName = "Alerts", Order = 1)]
		public string AlertSound
		{ get; set; }
       
		
		
		//[NinjaScriptProperty]
		[Display(Description = "Enable alerts at trend change",
		 ResourceType = typeof(Custom.Resource), Name = "Alert Enabled", GroupName = "Alerts", Order = 0)]
		public bool AlertEnabled
		{ get; set; }
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private TCTrendingTSF[] cacheTCTrendingTSF;
		public TCTrendingTSF TCTrendingTSF(int tSFPeriod, int tSFForecast, NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF display, int countertrendOpacity)
		{
			return TCTrendingTSF(Input, tSFPeriod, tSFForecast, display, countertrendOpacity);
		}

		public TCTrendingTSF TCTrendingTSF(ISeries<double> input, int tSFPeriod, int tSFForecast, NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF display, int countertrendOpacity)
		{
			if (cacheTCTrendingTSF != null)
				for (int idx = 0; idx < cacheTCTrendingTSF.Length; idx++)
					if (cacheTCTrendingTSF[idx] != null && cacheTCTrendingTSF[idx].TSFPeriod == tSFPeriod && cacheTCTrendingTSF[idx].TSFForecast == tSFForecast && cacheTCTrendingTSF[idx].Display == display && cacheTCTrendingTSF[idx].CountertrendOpacity == countertrendOpacity && cacheTCTrendingTSF[idx].EqualsInput(input))
						return cacheTCTrendingTSF[idx];
			return CacheIndicator<TCTrendingTSF>(new TCTrendingTSF(){ TSFPeriod = tSFPeriod, TSFForecast = tSFForecast, Display = display, CountertrendOpacity = countertrendOpacity }, input, ref cacheTCTrendingTSF);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.TCTrendingTSF TCTrendingTSF(int tSFPeriod, int tSFForecast, NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF display, int countertrendOpacity)
		{
			return indicator.TCTrendingTSF(Input, tSFPeriod, tSFForecast, display, countertrendOpacity);
		}

		public Indicators.TCTrendingTSF TCTrendingTSF(ISeries<double> input , int tSFPeriod, int tSFForecast, NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF display, int countertrendOpacity)
		{
			return indicator.TCTrendingTSF(input, tSFPeriod, tSFForecast, display, countertrendOpacity);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.TCTrendingTSF TCTrendingTSF(int tSFPeriod, int tSFForecast, NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF display, int countertrendOpacity)
		{
			return indicator.TCTrendingTSF(Input, tSFPeriod, tSFForecast, display, countertrendOpacity);
		}

		public Indicators.TCTrendingTSF TCTrendingTSF(ISeries<double> input , int tSFPeriod, int tSFForecast, NinjaTrader.NinjaScript.Indicators.TCTrendingTSF.DisplayTypeTCTrendingTSF display, int countertrendOpacity)
		{
			return indicator.TCTrendingTSF(input, tSFPeriod, tSFForecast, display, countertrendOpacity);
		}
	}
}

#endregion
